package websocket

import (
	"fmt"
	"testing"

	"gitlab.dms3.io/p2p/go-p2p-core/crypto"
	"gitlab.dms3.io/p2p/go-p2p-core/peer"

	csms "gitlab.dms3.io/p2p/go-conn-security-multistream"
	sec "gitlab.dms3.io/p2p/go-p2p-core/sec"
	"gitlab.dms3.io/p2p/go-p2p-core/sec/insecure"
)

var peerName = "PeerA"
var peerA peer.ID
var privA crypto.PrivKey
var peerB peer.ID
var privB crypto.PrivKey

func getPeer(name string) (peer.ID, crypto.PrivKey, error) {
	id := peerA
	priv := privA
	if name != peerName {
		id = peerB
		priv = privB
	}

	if len(id) == 0 {
		return id, priv, fmt.Errorf("missing peer ID, call initPeerId first")
	}

	_, err := priv.Raw()
	if err != nil {
		return id, priv, fmt.Errorf("mising priv key, call initPeerId first")
	}

	return id, priv, nil
}

func initPeerID() {
	ska, pa, err := crypto.GenerateKeyPair(crypto.Ed25519, 256)
	if err != nil {
		fmt.Printf("GenerateKeyPair failed %v", err)
	}
	ida, err := peer.IDFromPublicKey(pa)
	if err != nil {
		fmt.Printf("IDFromPublicKey failed %v", err)
	}

	skb, pb, err := crypto.GenerateKeyPair(crypto.Ed25519, 256)
	if err != nil {
		fmt.Printf("GenerateKeyPair failed %v", err)
	}
	idb, err := peer.IDFromPublicKey(pb)
	if err != nil {
		fmt.Printf("IDFromPublicKey failed %v", err)
	}

	peerA = ida
	privA = ska
	peerB = idb
	privB = skb
}

func makeInsecureTransport(t *testing.T, name string) (sec.SecureMuxer, peer.ID) {
	id := peerA
	if len(id) == 0 {
		initPeerID()
	}

	id, priv, err := getPeer(name)
	if err != nil {
		t.Fatal(err)
	}
	secMuxer := new(csms.SSMuxer)
	secMuxer.AddTransport(insecure.ID, insecure.NewWithIdentity(id, priv))
	return secMuxer, id
}
